Option Compare Binary
Option Explicit On 
Option Strict On

Imports System
Imports System.Windows.Forms

''' <summary>
'''   Stellt Funktionalitt zum Enumerieren aller Steuerelemente bestimmter Typen
'''   in einer Menge von Containern bereit, die nicht in einem der in einer weiteren
'''   Menge von Containern liegenden Container enthalten sind.
''' </summary>
Public Class ControlEnumerator

    ''' <summary>
    '''   Wird fr jedes whrend des Enumerationsvorgangs gefundene Steuerelement
    '''   ausgelst.
    ''' <summary>
    ''' <param name="sender">Die Quelle des Ereignisses.</param>
    ''' <param name="e">Ein <c>ControlFoundEventArgs</c>-Objekt, das die Ereignisdaten enthlt.</param>
    Public Event ControlFound(ByVal sender As Object, ByVal e As ControlFoundEventArgs)

    ' Private Variablen zum Speichern der Eigenschaftswerte.
    Private m_ExcludedContainers() As Control
    Private m_InterestingControlTypes() As Type
    Private m_RootContainers() As Control

    ''' <summary>
    '''   Ruft fr jedes Steuerelement, das einem der Steuerelemente in <c>RootContainers</c>
    '''   untergeordnet ist, jedoch nicht <c>ExcludedContainers</c>, und zustzlich noch einen
    '''   der Datentypen in <c>InterestingControlTypes</c> aufweist, das Ereignis
    '''   <c>ControlFound</c> auf.
    ''' </summary>
    Public Sub Enumerate()
        For Each ctr As Control In Me.RootContainers
            DoEnumeration(ctr)
        Next ctr
    End Sub

    ''' <summary>
    '''   Fhrt eine rekursive Enumeration der interessanten Steuerelemente in einem
    '''   Container durch.
    ''' </summary>
    ''' <param name="Control">Steuerelement, fr das die Enumeration aufgerufen wird.</param>
    Private Sub DoEnumeration(ByVal Control As Control)

        ' Der Container ist nicht einer der "verbotenen" Container.
        If Not IsExcludedContainer(Control) Then

            ' Es handelt sich nicht um einen Wurzelcontainer (diese sind als
            ' "Ergebniskandidaten" ausgenommen).
            If Not IsRootContainer(Control) Then
                If IsInterestingControl(Control) Then
                    RaiseEvent ControlFound(Me, New ControlFoundEventArgs(Control))
                End If
            End If

            ' Wenn der Container berhaupt Kinder besitzt.
            If Control.HasChildren Then

                ' Fr alle Steuerelemente rekursiv aufrufen.
                For Each ctr As Control In Control.Controls
                    DoEnumeration(ctr)
                Next ctr
            End If
        End If
    End Sub

    ''' <summary>
    '''   Gibt zurck, ob das angegeben Steuerelement in der Sammlung der Wurzelcontainer
    '''   enthalten ist.
    ''' </summary>
    ''' <param name="Control">Zu untersuchendes Steuerelement.</param>
    ''' <returns>
    '''   Boolescher Wert, der angibt, ob das im Parameter <paramref name="Control"/>
    '''   bergebene Steuerelement ein Wurzelcontainer ist.
    ''' </returns>
    Private Function IsRootContainer(ByVal Control As Control) As Boolean
        For Each ctr As Control In Me.RootContainers
            If Control Is ctr Then
                Return True
            End If
        Next ctr
        Return False
    End Function

    ''' <summary>
    '''   Gibt zurck, ob es sich beim im Parameter <paramref name="Control"/> bergebenen
    '''   Steuerelement um einen von der Enumeration ausgenommenen Container handelt.
    ''' </summary>
    ''' <param name="Control">Zu untersuchendes Steuerelement.</param>
    ''' <returns>
    '''   Boolescher Wert, der angibt, ob das im Parameter <paramref name="Control"/>
    '''   bergebene Steuerelement eine uninteressanter Container ist.
    ''' </returns>
    Private Function IsExcludedContainer(ByVal Control As Control) As Boolean
        For Each ctr As Control In Me.ExcludedContainers
            If Control Is ctr Then
                Return True
            End If
        Next ctr
        Return False
    End Function

    ''' <summary>
    '''   Gibt zurck, ob das im Parameter <paramref name="Control"/> angegebene Steuerelement
    '''   Instanz eines interessanten Typs ist.
    ''' </summary>
    ''' <param name="Control">Zu untersuchendes Steuerelement.</param>
    ''' <returns>
    '''   Boolescher Wert, der angibt, ob das im Parameter <paramref name="Control"/>
    '''   bergebene Steuerelement interessant ist.
    ''' </returns>
    Private Function IsInterestingControl(ByVal Control As Control) As Boolean
        For Each typ As Type In Me.InterestingControlTypes
            If Control.GetType() Is typ Then
                Return True
            End If
        Next typ
        Return False
    End Function

    ''' <summary>
    '''   Gibt eine Menge von Steuerelementcontainern, in denen nicht weiter gesucht werden soll,
    '''   an oder gibt sie zurck.
    ''' </summary>
    ''' <value>Menge von Containern, deren enthaltene Steuerelemente nicht betrachtet werden.</value>
    Public Property ExcludedContainers() As Control()
        Get
            Return m_ExcludedContainers
        End Get
        Set(ByVal Value() As Control)
            m_ExcludedContainers = Value
        End Set
    End Property

    ''' <summary>
    '''   Gibt eine Menge interessanter Steuerelementtypen an oder gibt sie zurck. Steuerelemente
    '''   eines dieser Typen, die whrend der Enumeration durchlaufen werden, werden ausgegeben.
    ''' </summary>
    ''' <vaule>Menge interessanter Steuerelementtypen.</value>
    Public Property InterestingControlTypes() As Type()
        Get
            Return m_InterestingControlTypes
        End Get
        Set(ByVal Value() As Type)
            m_InterestingControlTypes = Value
        End Set
    End Property

    ''' <summary>
    '''   Gibt eine Menge von Wurzelcontainern, die auf untergeordnete Steuerelemente eines
    '''   der Typen in <c>InterestingControlTypes</c> untersucht werden soll, an oder gibt
    '''   sie zurck.
    ''' </summary>
    ''' <value>Menge von zu durchsuchenden Wurzelcontainern.</value>
    Public Property RootContainers() As Control()
        Get
            Return m_RootContainers
        End Get
        Set(ByVal Value() As Control)
            m_RootContainers = Value
        End Set
    End Property
End Class